#include "HwDevice.h"

// Sample for adding emulation of a HW device to Q-emuLator by building a Windows DLL.
// This sample QL device has a 1-byte memory (useful, huh?), writing to $18100 stores the
// value in it, reading at $18101 returns the last value stored.
// DLL installation: In the directory containing QemuLator.exe, create a subdirectory
// named Extensions and place the DLL in it.

#define ADDR_BASE 0x18100
#define ADDR_WIDTH 2
#define ADDR_WRITE   ADDR_BASE
#define ADDR_READ (ADDR_BASE+1)

static unsigned char g_byte; 

// Handle memory-mapped write to address addr
// Return 1 if address was for this device, 0 otherwise
HW_EXT_API int HW_WriteReg(unsigned int addr, unsigned char b)
{
	int result = 1;

	switch (addr){

	case ADDR_WRITE:
		g_byte = b;
		break;

	default:
		result = 0;
		break;
	}

	return result;
}

// Handle memory-mapped read from address addr
// Return 1 if address was for this device, 0 otherwise
HW_EXT_API int HW_ReadReg(unsigned int addr, unsigned char *pVal)
{
	int result = 1;
	*pVal = 0;
  
	switch (addr) {

	case ADDR_READ:
		*pVal = g_byte;
		break;

	default:
		result = 0;
		break;
	}

	return result;
}

// Called when DLL is loaded
HW_EXT_API int HW_Allocate(struct IHwHost *pHost, int suggestedPeripheralID)
{
	// Mandatory: define address range.
	// If emulator doesn't accept range (e.g. conflict with other HW), extension can try a different range.
	// For peripherals above $C0000, suggesterdPeripheralID is the the suggested 'slot' the peripheral should
	// occupy. Ignore for peripherals in $C000 - $20000 range.
	if (!pHost->SetAddressRange(ADDR_BASE, ADDR_WIDTH))
		return 0;

	// Possibly more initialization here

	return 1;
}

// Called before DLL is unloaded (optional)
HW_EXT_API void HW_Deallocate()
{
}

// Called when emulated QL starts (optional)
// Note: for normal start, bReset will be 0.
// When the QL is restarted while running, emulator will call HW_Stop followed by HW_Start
// and bReset will be 1 for both.
HW_EXT_API void HW_Start(int bReset)
{
	g_byte = 0;
}

// Called when emulated QL stops (optional)
HW_EXT_API void HW_Stop(int bReset)
{
}
